------------------------------------
-- Shot Glass Raid Frames
------------------------------------
local addonname = "ShotGlass"
local artpath = "Interface\\Addons\\ShotGlass\\Art\\"
local rangePollFrequency = .6

-- Clique Support
ClickCastFrames = ClickCastFrames or {}											

-- Function List
local UpdateButton, OnShowButton, OnChangeAttribute, OnEvent, SetHandlers		
local CreateGroupHeader, CreateUnitFrameArtwork
local CallArtworkFunction, SecureInitFunction

------------------------------------
-- Aura Tracking List
------------------------------------
local TrackedAuras = { 
	-- Example Aura
	Rejuvenation = {position = 1, color = {r = .8,g = 0, b = .8, a = 1},},
	-- Or, if you know the spell ID...
	[GetSpellInfo(774)] = {position = 1, color = {r = .8,g = 0, b = .8, a = 1},},
	-- More Auras
	Regrowth = {position = 2, color = {r = .2,g = 1, b = .4, a = 1},},
	Lifebloom = {position = 3, color = {r = .6,g = 1, b = .1, a = 1},},
	["Wild Growth"] = {position = 3, color = {r = .2,g = 1, b = 1, a = 1},},
	Thorns = {position = 3, color = {r = .7,g = .5, b = .3, a = 1},},
	-- Debuff Types
	Poison = {position = 5, color = {r = 0,g = .9, b = .1, a = 1},},
	Curse = {position = 4, color = {r = .9,g = 0, b = .7, a = 1},},
	Magic = {position = 6, color = {r = 0,g = .3, b = .9, a = 1},},
	--Disease = {position = "Indicator4", color = {r = .7,g = .7, b = 1, a = 1},},
	
	-- You can also display harmful auras...
	-- Rotface
	["Mutated Infection "] = {position = 4, color = {r = 1,g = 0, b = 0, a = 1},},
	-- Sindragosa 
	["Ice Tomb"] = {position = 4, color = {r = .4,g = .4, b = 1, a = 1},},
	["Frost Beacon"] = {position = 4, color = {r = 1,g = 1, b = 1, a = 1},},
	["Unchained Magic"] = {position = 5, color = {r = .5,g = 0, b = 1, a = 1},},
	["Mystic Buffet"] = {position = 6, color = {r = .5,g = 0, b = .5, a = 1},},
	-- Blood Queen
	["Essence of the Blood Queen"] = {position = 5, color = {r = 1,g = 0, b = 0, a = 1},},
	["Frenzied Bloodthirst"] = {position = 5, color = {r = .8,g = .3, b = 0, a = 1},},
}

------------------------------------
-- Helper Functions
------------------------------------

-- Abbreviates the Unit Name
local function TruncateName(text) if text then return text:gsub("(%u%l%l%l%l)(.*)", "%1") end end

-- Resets all Indicators
local function ResetIndicators(self) 
	local name, indicator
	for name, indicator in pairs(self.Graphics.Indicators) do 
		if type(indicator)=='table' then indicator:Hide() end
	end
end

-- Polls for Range Check
local function PollOnUpdate(self)
	local _, index, name, count, expiration, mine, auratarget
	local unitid = self.UnitID
	if not unitid then return end

	local now = GetTime()
	if now > self.rangeUpdateTime then self.rangeUpdateTime = now + rangePollFrequency else return end
	
	-- Range Check
	if UnitInRange(unitid) then self.Graphics:SetScale(1)
	else self.Graphics:SetScale(.6) end
end

-- Hides Party Frames
for i=1,4 do
	local frame = _G["PartyMemberFrame"..i]
	frame:UnregisterAllEvents()
	frame.Show = function() end
	frame:Hide()
end

------------------------------------
-- Events
------------------------------------
local UnitEventHandlers = {}

function UnitEventHandlers.UNIT_DISPLAYPOWER(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then 
		local _, powertype = UnitPowerType(unitid or "")
		if powertype == "MANA" then
			self.Graphics.Power:Show()
			self.Graphics.Power:SetMinMaxValues(0,UnitPowerMax(unitid))
			self.Graphics.Power:SetValue(UnitPower(unitid))
		else
			self.Graphics.Power:Hide()
		end
	end
end

function UnitEventHandlers.UNIT_POWER(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then 
		self.Graphics.Power:SetValue(UnitPower(unitid))
	end
end

function UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unit)
	
	local unitid = self.UnitID
	if not unitid then return end
	if unit == unitid then
		if UnitIsDead(unitid) or UnitIsGhost(unitid) or (not UnitIsConnected(unitid))then 
			self.Graphics.Health:SetAlpha(.7)
		else 
			self.Graphics.Health:SetAlpha(1)
		end
	end
end
UnitEventHandlers.PARTY_MEMBER_ENABLE = UnitEventHandlers.PARTY_MEMBER_DISABLE
UnitEventHandlers.PLAYER_DEAD = UnitEventHandlers.PARTY_MEMBER_DISABLE

function UnitEventHandlers.UNIT_HEALTH(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	--[[
	if UnitHasVehicleUI(unit) then
		self.Graphics.Health:SetMinMaxValues(0,UnitHealthMax(unitid))
		self.Graphics.Health:SetValue(UnitHealth(unitid))
	end
	--]]
	
		
	if unit == unitid then 
			self.Graphics.Health:SetMinMaxValues(0,UnitHealthMax(unitid))
			self.Graphics.Health:SetValue(UnitHealth(unitid))			
	end
end
UnitEventHandlers.UNIT_MAXHEALTH = UnitEventHandlers.UNIT_HEALTH

function UnitEventHandlers.PLAYER_TARGET_CHANGED(self) 
	local unitid = self.UnitID

	if not unitid then return end
	if UnitIsUnit("target", unitid) then self.Graphics.Border:Show()
	else self.Graphics.Border:Hide() end
end

function UnitEventHandlers.UNIT_THREAT_SITUATION_UPDATE(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	local unitaggro = UnitThreatSituation(unitid)
	if unitaggro and unitaggro > 1 then 
		self.Graphics.Aggro:Show() 
	else 
		self.Graphics.Aggro:Hide() 
	end
end

function UnitEventHandlers.UNIT_AURA(self, unit)
	local index, name, debuffType, caster , stacks, _
	local unitid = self.UnitID
	if not unitid then return end
	
	if unit == unitid then 
		ResetIndicators(self)
		-- Check Buffs
		for index = 1, 40 do
			name, _, _, stacks, _, _, _, caster = UnitAura(unitid,index,"PLAYER")
			if not name then break end
			auratarget = TrackedAuras[name] 
			if auratarget then 
				
				local color = auratarget.color
				-- Color
				self.Graphics.Indicators[auratarget.position].Image:SetVertexColor(color.r,color.g,color.b,color.a)
				-- Stack Text
				if (stacks > 0) and (caster == "player") then self.Graphics.Indicators[auratarget.position].Text:SetText(stacks) 
				else self.Graphics.Indicators[auratarget.position].Text:SetText("") end
				-- Show
				self.Graphics.Indicators[auratarget.position]:Show()	
			end
		end
		-- Check Debuffs
		for index = 1, 40 do
			name, _, _, _, debuffType = UnitDebuff(unitid,index)
			if not name then break end
			auratarget = TrackedAuras[name] or TrackedAuras[debuffType]
			if auratarget then 
				local color = auratarget.color
				self.Graphics.Indicators[auratarget.position].Image:SetVertexColor(color.r,color.g,color.b,color.a)
				self.Graphics.Indicators[auratarget.position]:Show()
			end
		end
		
	end
end
-- UNIT_FLAGS - Stunned, Feared, etc.  -- disabled, health, aura

function UnitEventHandlers.UNIT_PORTRAIT_UPDATE(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	UnitEventHandlers.UNIT_DISPLAYPOWER(self, unitid)
	UnitEventHandlers.UNIT_AURA(self, unitid)
	UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)	
	UnitEventHandlers.UNIT_HEALTH(self, unitid)
end
UnitEventHandlers.PLAYER_UNGHOST = UnitEventHandlers.UNIT_PORTRAIT_UPDATE
UnitEventHandlers.PLAYER_ALIVE = UnitEventHandlers.UNIT_PORTRAIT_UPDATE

function UnitEventHandlers.UNIT_FLAGS(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	UnitEventHandlers.UNIT_AURA(self, unitid)
	UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)
	
end

function UnitEventHandlers.UNIT_ENTERED_VEHICLE(self, unit)
	OnChangeAttribute(self, "unit")
	if self.UnitID then UpdateButton(self, self.UnitID) end
end
UnitEventHandlers.UNIT_EXITED_VEHICLE = UnitEventHandlers.UNIT_ENTERED_VEHICLE
UnitEventHandlers.UNIT_PET = UnitEventHandlers.UNIT_ENTERED_VEHICLE
--UnitEventHandlers.PLAYER_GAINS_VEHICLE_DATA = UnitEventHandlers.UNIT_ENTERED_VEHICLE
--UnitEventHandlers.PLAYER_LOSES_VEHICLE_DATA = UnitEventHandlers.UNIT_ENTERED_VEHICLE
--UnitEventHandlers.VEHICLE_UPDATE = UnitEventHandlers.UNIT_ENTERED_VEHICLE

function UnitEventHandlers.UNIT_HEAL_PREDICTION(self, unit)
	local unitid = self.UnitID
	if not unitid then return end
	
	if unit == unitid then
		local incomingHeal = UnitGetIncomingHeals(unitid) or 0
		if incomingHeal > 0 then incomingHeal = UnitHealth(unitid)+incomingHeal end
		self.Graphics.IncomingHeal:SetValue(incomingHeal)	
	end
end

------------------------------------
-- Update Functions
------------------------------------
function UpdateButton(self, unitid)
		local class, color, Graphics
		--if not unitid then return end
		
		self.Nextupdate = 0
		Graphics = self.Graphics
		-- Colors
		class = select(2, UnitClass(unitid)) or "WARRIOR"
		color = RAID_CLASS_COLORS[class]
		-- Name
		Graphics.Text:SetText(TruncateName(UnitName(unitid)))
		Graphics.Text:SetTextColor(color.r*.5, color.g*.5, color.b*.5, 1)
		-- Health
		local maxhealth = UnitHealthMax(unitid)
		Graphics.Health:SetStatusBarColor(0.05, 0.05, 0.05, 1)
		Graphics.Health:SetMinMaxValues(0,maxhealth)
		Graphics.Health:SetBackdropColor(color.r, color.g, color.b, 1)
		Graphics.IncomingHeal:SetStatusBarColor(0.05, 0.05, 0.05, .6)
		Graphics.IncomingHeal:SetMinMaxValues(0,maxhealth)
		-- Power		
		UnitEventHandlers.UNIT_DISPLAYPOWER(self, unitid)
		UnitEventHandlers.UNIT_AURA(self, unitid)
		UnitEventHandlers.PARTY_MEMBER_DISABLE(self, unitid)	
		UnitEventHandlers.UNIT_HEALTH(self, unitid)	
		UnitEventHandlers.UNIT_HEAL_PREDICTION(self, unitid)	
end

function SetHandlers(self)
	self:SetScript("OnShow", OnShowButton)
	self:SetScript("OnEvent", OnEvent)
	self:SetScript("OnUpdate", PollOnUpdate)
	for eventname, _ in pairs(UnitEventHandlers) do self:RegisterEvent(eventname) end
end

function OnShowButton(self)
	if not InCombatLockdown() then self:SetAttribute('*type1','target') end
	
	local unitid = self:GetAttribute("unit")
	if unitid then 
		self.UnitID = unitid
		UpdateButton(self, unitid)
	end
end

function OnEvent(self, event, ...)
	UnitEventHandlers[event](self, ...) 
end

function OnChangeAttribute(self, attribute, value)
	if attribute == "_wrapentered" then return	-- Clique filter
	elseif attribute == "unit" then 
		if not InCombatLockdown() then 
			self:SetAttribute('*type1','target')
			--self:SetAttribute('type','target')
			--self:SetAttribute("toggleForVehicle", true)
			--self:SetAttribute("allowVehicleTarget", true)
		end
		
		ClickCastFrames[self] = true		-- Clique
		SetHandlers(self)
		
		local unitid = self:GetAttribute("unit")
		if unitid then 
			self.UnitID = unitid
			UpdateButton(self, unitid) 
		end
	end
end

------------------------------------
-- Unit Frame Artwork
------------------------------------
function CreateUnitFrameArtwork(self)
	local Graphics
	self:EnableMouse(true)
	self.info = {}
	
	-- Create Base Frames
	self.AuraWatcher = CreateFrame("Frame", nil, self)
	self.Graphics = CreateFrame("Frame", nil, self)
	self.auraUpdateTime = 0
	self.rangeUpdateTime = 0
	
	Graphics = self.Graphics
	Graphics:SetPoint("CENTER", self)
	Graphics:SetWidth(64)
	Graphics:SetHeight(64)
		
	-- Health Bar 
	Graphics.Health = CreateFrame("StatusBar", nil, Graphics)
	Graphics.Health:SetStatusBarTexture(artpath.."HealthStatusbar")
	Graphics.Health:SetBackdrop({bgFile = artpath.."HealthStatusbar"});
	Graphics.Health:SetOrientation("VERTICAL")
	Graphics.Health:SetPoint("CENTER", 0, -1)
	Graphics.Health:SetFrameLevel(Graphics:GetFrameLevel()-1)
	Graphics.Health:SetWidth(36)
	Graphics.Health:SetHeight(34)
	
	-- Heal Prediction Bar 
	Graphics.IncomingHeal = CreateFrame("StatusBar", nil, Graphics.Health)
	Graphics.IncomingHeal:SetStatusBarTexture(artpath.."HealthStatusbar")
	Graphics.IncomingHeal:SetOrientation("VERTICAL")
	Graphics.IncomingHeal:SetAllPoints(Graphics.Health)
	
	-- Overlay Anchor (To make layering easier)
	Graphics.OverlayAnchor = CreateFrame("StatusBar", nil, Graphics.Health)
	Graphics.OverlayAnchor:SetAllPoints()
	Graphics.OverlayAnchor:SetFrameLevel(Graphics.IncomingHeal:GetFrameLevel()+1)
	
	-- Power (Mana) Bar
	Graphics.Power = CreateFrame("StatusBar", nil, Graphics.OverlayAnchor)
	Graphics.Power:SetStatusBarTexture(artpath.."ManaStatusbar")
	Graphics.Power:SetBackdrop({bgFile = artpath.."ManaStatusbar"});
	Graphics.Power:SetOrientation("VERTICAL")
	Graphics.Power:SetPoint("CENTER", 16, 7)
	Graphics.Power:SetHeight(28)	
	Graphics.Power:SetWidth(7)	
	Graphics.Power:SetBackdropColor(0, 0, 0, 1)
	Graphics.Power:SetStatusBarColor(4/255, 78/255, 141/255, 1)

	Graphics.Power.Border = Graphics.Power:CreateTexture(nil, "OVERLAY")
	Graphics.Power.Border:SetTexture(artpath.."ManaOverlay")
	Graphics.Power.Border:SetPoint("TOPLEFT", -1,1)	
	Graphics.Power.Border:SetHeight(32)	
	Graphics.Power.Border:SetWidth(16)	

	-- Create Name Text Field
	Graphics.Text = Graphics.OverlayAnchor:CreateFontString(nil, "OVERLAY")
	Graphics.Text:SetFont(artpath.."LiberationSans-Regular.ttf", 10)
	Graphics.Text:SetPoint("BOTTOM", Graphics.Health, "BOTTOM", 0, 1)
	Graphics.Text:SetWidth(50)
	Graphics.Text:SetHeight(13)
	Graphics.Text:SetShadowColor(0,0,0,1) 
	Graphics.Text:SetShadowOffset(1,-1) 
	
	-- Create Mouseover Highlight
	Graphics.Glow = Graphics.OverlayAnchor:CreateTexture(nil, "HIGHLIGHT")
	Graphics.Glow:SetTexture("Interface/TARGETINGFRAME/UI-StatusBar")
	Graphics.Glow:SetVertexColor(.5,.5,.5,.5)
	Graphics.Glow:SetBlendMode("ADD")
	Graphics.Glow:SetAllPoints(Graphics.Health)

	-- Overlay Artwork
	Graphics.Overlay = Graphics.OverlayAnchor:CreateTexture(nil, "ARTWORK")
	Graphics.Overlay:SetTexture(artpath.."HealthOverlay")
	Graphics.Overlay:SetAllPoints(Graphics)
	
	-- Border/Selection Box Artwork
	Graphics.Border = Graphics:CreateTexture(nil, "ARTWORK")
	Graphics.Border:SetTexture(artpath.."Select")
	Graphics.Border:SetAllPoints()
	Graphics.Border:Hide()

	Graphics.Indicators = {}
	local Indicators = Graphics.Indicators
	
	-- Indicators Table
	Graphics.IndicatorLayer = CreateFrame("Frame", nil, Graphics)
	Graphics.IndicatorLayer:SetAllPoints(Graphics)
	Graphics.IndicatorLayer:Show()
	
	local Indicator, IndicatorIndex
	for index = 1, 6 do
		Indicators[index] = CreateFrame("Frame", nil, Graphics.IndicatorLayer)
		Indicator = Indicators[index]
		Indicator:SetWidth(16)
		Indicator:SetHeight(16)
		Indicator.Image = Indicator:CreateTexture(nil, "ARTWORK")
		Indicator.Image:SetAllPoints()
		Indicator.Text = Indicator:CreateFontString(nil, "OVERLAY")
		Indicator.Text:SetFont(artpath.."LiberationSans-Regular.ttf", 7, "OUTLINE")
		Indicator.Text:SetPoint("TOP", Indicator, "BOTTOM", 1, 7)
		Indicator.Text:SetWidth(12)
		Indicator.Text:SetHeight(12)
		Indicator:Hide()	
	end
	
	for index = 1, 3 do Indicators[index].Image:SetTexture(artpath.."DebuffDot") end
	for index = 2, 3 do Indicators[index]:SetPoint("RIGHT", Indicators[index-1], "CENTER", -1, 0) end
	Indicators[1]:SetPoint("TOP", 6, -7)
	
	for index = 4, 6 do Indicators[index].Image:SetTexture(artpath.."DebuffArrow") end
	for index = 5, 6 do Indicators[index]:SetPoint("RIGHT", Indicators[index-1], "CENTER", -3, 0) end
	Indicators[4]:SetPoint("BOTTOM", 11, 4)
	
	-- Aggro/Outline Artwork
	Graphics.Aggro = Graphics:CreateTexture(nil, "OVERLAY")
	Graphics.Aggro:SetTexture(artpath.."Aggro")
	Graphics.Aggro:SetVertexColor(1,0,0,1)
	Graphics.Aggro:SetAllPoints()
	Graphics.Aggro:Hide()
	
	-- Scripts
	self:SetScript("OnAttributeChanged", OnChangeAttribute)
end

-- Calls the Unit Frame Artwork function; Called by the secure config function
function CallArtworkFunction(self, frame, ...)
	CreateUnitFrameArtwork(_G[frame])
end

-- Initialization Function: Set Size and other properties of the Unit Button
-- Variables from addons are not accessible when this function is called.
SecureInitFunction = [[
	local header = self:GetParent()
	header:CallMethod('CallArtworkFunction', self:GetName())
	self:SetHeight(45)
	self:SetWidth(45)
]]

-- Creates the Group Header, which dynamically controls the creation of Unit Frames
function CreateGroupHeader()
	local RaidGroupHeader = CreateFrame("Frame", addonname.."_GroupHeader", UIParent, "SecureGroupHeaderTemplate")
	local HandleFrame = CreateFrame("Frame", addonname.."_Handle", RaidGroupHeader)
	
	RaidGroupHeader:SetPoint("CENTER", UIParent)
	RaidGroupHeader:SetMovable(true)
	RaidGroupHeader:SetClampedToScreen(true)

	-- RaidGroupHeader Frame Header
	RaidGroupHeader:SetAttribute("template", "SecureUnitButtonTemplate")
	RaidGroupHeader:SetAttribute("templateType", "Button")
	RaidGroupHeader:SetAttribute("groupFilter", "1,2,3,4,5,6,7,8")
	RaidGroupHeader:SetAttribute("showRaid", true)
	RaidGroupHeader:SetAttribute("showPlayer", true)
	RaidGroupHeader:SetAttribute("showSolo", true)
	RaidGroupHeader:SetAttribute("point","RIGHT")
	RaidGroupHeader:SetAttribute("unitsPerColumn",5)
	RaidGroupHeader:SetAttribute("maxColumns",8)
	RaidGroupHeader:SetAttribute("columnAnchorPoint","TOP")
	RaidGroupHeader.CallArtworkFunction = CallArtworkFunction	-- This function is called from the init function
	RaidGroupHeader:SetAttribute("initialConfigFunction", SecureInitFunction) 		-- Init function (Secure)
			
	-- Handle Frame
	HandleFrame:SetPoint("TOP", RaidGroupHeader, "TOP", 0, 0)
	HandleFrame:SetPoint("RIGHT", RaidGroupHeader, "RIGHT", 18, 0)
	HandleFrame:SetWidth(16)
	HandleFrame:SetHeight(16)

	HandleFrame:EnableMouse(true)	-- /run ShotGlassRaidFrames.HandleFrame:EnableMouse(false) -- to lock the frame, in game
	HandleFrame:SetScript("OnMouseDown",function(self, button) RaidGroupHeader:StartMoving() end)
	HandleFrame:SetScript("OnMouseUp",function(self, button) RaidGroupHeader:StopMovingOrSizing() end)
	
	HandleFrame.Texture = HandleFrame:CreateTexture(nil, "ARTWORK")
	HandleFrame.Texture:SetTexture(artpath.."CornerHandle")
	HandleFrame.Texture:SetAllPoints(HandleFrame)
	RaidGroupHeader.HandleFrame = HandleFrame
	
	RaidGroupHeader:Show()
	return RaidGroupHeader
end

ShotGlassRaidFrames = CreateGroupHeader()
ShotGlassRaidFrames.TrackedAuras = TrackedAuras